package net.wanji.datacenter.kafka;

import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.utils.tool.StringUtils;
import net.wanji.datacenter.pojo.vo.GreenBeltOptInfoVO;
import net.wanji.datacenter.service.GreenBeltInfoService;
import net.wanji.datacenter.util.JsonFileWriter;
import org.apache.kafka.clients.consumer.Consumer;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.springframework.kafka.annotation.KafkaListener;
import org.springframework.kafka.listener.KafkaListenerErrorHandler;
import org.springframework.kafka.listener.ListenerExecutionFailedException;
import org.springframework.kafka.support.Acknowledgment;
import org.springframework.messaging.Message;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;

/**
 * @author duanruiming
 * @date 2024/11/20 19:12
 */
@Component
@Slf4j
public class GreenBeltDataConsumerHandler implements KafkaListenerErrorHandler {

    @Resource
    private GreenBeltInfoService greenBeltInfoService;
    @Resource
    private JsonFileWriter jsonFileWriter;


    /**
     * 卫博发送绿波时序图数据
     * @param record
     * @param acknowledgment
     * @throws Exception
     */
    @KafkaListener(topics = {"${kafka-consumer.greenBeltInfoDataTopic}"}, groupId = "group")
    public void receiveGreenBeltInfoData(ConsumerRecord<Object, Object> record, Acknowledgment acknowledgment) throws Exception {
        try {
            String message = String.valueOf(record.value());
            jsonFileWriter.writeJsonToFile(message);
            if (StringUtils.isNotBlank(message)) {
                GreenBeltOptInfoVO greenBeltOptInfoVO = greenBeltInfoService.convertData(message);
                greenBeltInfoService.save(greenBeltOptInfoVO);
            }
            acknowledgment.acknowledge();
        } catch (Exception e) {
            log.error("绿波GreenOpt实时数据转换异常", e);
            throw new Exception(e);
        }
    }

    /**
     * 绿波周期数据
     * @param record
     * @param acknowledgment
     * @throws Exception
     */
    @KafkaListener(topics = {"${kafka-consumer.greenWavePeriodTopic}"}, groupId = "group")
    public void receiveGreenPeriodData(ConsumerRecord<Object, Object> record, Acknowledgment acknowledgment) throws Exception {
        String originalData = String.valueOf(record.value());
        try {
            greenBeltInfoService.savePeriod(originalData);
        } catch (Exception e) {
            log.error("绿波GreenPeriod实时数据转换异常", e);
            throw new Exception();
        }
        acknowledgment.acknowledge();
    }

    @Override
    @NonNull
    public Object handleError(Message<?> message, ListenerExecutionFailedException e) {
        return new Object();
    }

    @Override
    @NonNull
    public Object handleError(Message<?> message, ListenerExecutionFailedException exception, Consumer<?, ?> consumer) {
        String errorMessage = String.format("监听主题：%s，消费者详情：%s，异常信息：%s，消息详情：%s",
                consumer.listTopics(), consumer.groupMetadata(), exception, message);
        log.error(errorMessage);
        return KafkaListenerErrorHandler.super.handleError(message, exception, consumer);
    }
}
