package net.wanji.datacenter.util;

import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * @author duanruiming
 * @date 2025/02/10 14:23
 */
@Slf4j
@Component
public class JsonFileWriter {
    /**
     * 设置文件大小200MB
     */
    private static final long MAX_FILE_SIZE = 200 * 1024 * 1024;
    /**
     * 服务器的存储路径
     */
    private static final String BASE_DIR = "/data/greenOptFile/";

    private File currentFile;
    private long currentFileSize = 0;

    public JsonFileWriter() throws Exception {
        createNewFile();
    }

    /**
     * 创建新的文件
     *
     * @throws Exception
     */
    private void createNewFile() throws Exception {
        String dateDirectory = new SimpleDateFormat("yyyy-MM-dd").format(new Date());
        String hourMinute = new SimpleDateFormat("HH-mm").format(new Date());
        Path dirPath = Paths.get(BASE_DIR, dateDirectory);
        try {
            // 确保目录存在
            Files.createDirectories(dirPath);
            String fileName = "greenOpt_" + hourMinute + ".json";
            currentFile = new File(dirPath.toFile(), fileName);
            currentFileSize = 0;
        } catch (Exception e) {
            log.error("创建文件失败：{}", e);
            throw new Exception(e);
        }
    }

    /**
     * 将 JSON 数据写入文件
     *
     * @param json
     */
    public void writeJsonToFile(String json) throws Exception {
        try {
            byte[] bytes = json.getBytes("UTF-8");

            // 如果当前文件大小加上新数据大于 200MB，创建新文件
            if (currentFileSize + bytes.length > MAX_FILE_SIZE) {
                createNewFile();
            }

            // 写入当前文件
            try (BufferedOutputStream bos = new BufferedOutputStream(new FileOutputStream(currentFile, true))) {
                bos.write(bytes);
                bos.flush();
                currentFileSize += bytes.length;
            }

        } catch (IOException e) {
            log.error("将数据写入文件失败：{}", e);
            throw new Exception(e);
        }
    }
}
